/* Copyright 2009, Unpublished Work of Technologic Systems
 * All Rights Reserved.
 *
 * THIS WORK IS AN UNPUBLISHED WORK AND CONTAINS CONFIDENTIAL,
 * PROPRIETARY AND TRADE SECRET INFORMATION OF TECHNOLOGIC SYSTEMS.
 * ACCESS TO THIS WORK IS RESTRICTED TO (I) TECHNOLOGIC SYSTEMS 
 * EMPLOYEES WHO HAVE A NEED TO KNOW TO PERFORM TASKS WITHIN THE SCOPE
 * OF THEIR ASSIGNMENTS AND (II) ENTITIES OTHER THAN TECHNOLOGIC
 * SYSTEMS WHO HAVE ENTERED INTO APPROPRIATE LICENSE AGREEMENTS.  NO
 * PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED, COPIED, 
 * DISTRIBUTED, REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED, 
 * EXPANDED, COLLECTED, COMPILED, LINKED, RECAST, TRANSFORMED, ADAPTED
 * IN ANY FORM OR BY ANY MEANS, MANUAL, MECHANICAL, CHEMICAL, 
 * ELECTRICAL, ELECTRONIC, OPTICAL, BIOLOGICAL, OR OTHERWISE WITHOUT
 * THE PRIOR WRITTEN PERMISSION AND CONSENT OF TECHNOLOGIC SYSTEMS.
 * ANY USE OR EXPLOITATION OF THIS WORK WITHOUT THE PRIOR WRITTEN
 * CONSENT OF TECHNOLOGIC SYSTEMS COULD SUBJECT THE PERPETRATOR TO
 * CRIMINAL AND CIVIL LIABILITY.
 */

/* The TS-4500 was released Q1 2010 and is comprised of a Cavium
 * CNS2132 250Mhz ARM9 CPU, Lattice XP2 5k FPGA, 64MB DDR SDRAM, SD
 * socket and 512MByte NAND flash chip.  
 *
 * FPGA functions include:
 *   - micro SD slot 
 *   - SPI controller (75Mhz - 2.5Mhz clock rate) with 4 chipselects .
 *   - 53 DIO pins on the TS-socket header
 *   - Processor bootstrap + TS-BOOTROM firmware store
 *   - I2C RTC interface
 *   - watchdog
 *   - 3 XUART channels
 *   - Improved SJA1000C CAN controller
 *   - storage of board MAC address and CPU speed mode in Lattice tagmem.
 *   - miscellaneous glue logic (leds, reset, etc..)
 *   
 * CPU bootstrap is accomplished by the FPGA emulating a SPI flash
 * chip.  TS-BOOTROM resides in a 16KByte initialized blockram in the
 * FPGA.
 *
 * WISHBONE bridge interface is accomplished with a custom SPI protocol
 * "SBUS" and has provision currently for 128 bytes of registers.
 *
 */

module ts4500_top( 
  fpga_25mhz_pad,

  dio_pad,

  /* spi interface */
  cpu_spi_clk_pad,
  cpu_spi_mosi_pad,
  cpu_spi_miso_pad,
  spi_clk_pad,
  spi_mosi_pad,
  spi_miso_pad,

  gpio_a0_pad,
  gpio_a1_pad,

  gpio_a3_pad,
  gpio_a15_pad,
  gpio_a16_pad,
  gpio_a17_pad,

  gpio_a28_pad,
  gpio_a29_pad,

  int28_pad,

  gpio_a22_pad,
  gpio_a23_pad,

  /* reset pins */
  un_reset_pad,
  wd_resetn_pad,

  /* sd interface */
  en_sd_power_pad,
  sd_d0_pad,
  sd_d1_pad,
  sd_d2_pad,
  sd_d3_pad,
  sd_cmd_pad,
  sd_clk_pad,

  /* led control pins */
  red_led_pad,
  green_led_pad,

  /* ethernet led control pins */
  eth_left_ledn_pad,
  eth_right_ledn_pad,

  /* rtc pins */
  rtc_sda_pad,
  rtc_scl_pad,

  nand_csn_pad,
  nand_rdn_pad,
  nand_wrn_pad,
  nand_ale_pad,
  nand_cle_pad,
  nand_busyn_pad,
  nand_d_pad
);

/* global signals */
input fpga_25mhz_pad;
inout [52:0] dio_pad;

/* spi interface */
input cpu_spi_clk_pad;
input cpu_spi_mosi_pad;
inout cpu_spi_miso_pad;
output spi_clk_pad;
output spi_mosi_pad;
input spi_miso_pad;

/* interrupts */
output gpio_a0_pad;
output gpio_a1_pad;

/* i2s */
input gpio_a3_pad;
input gpio_a15_pad;
input gpio_a16_pad;
input gpio_a17_pad;

input gpio_a28_pad;  //SPI_CS0
input gpio_a29_pad;  //SPI_CS1

inout int28_pad;

input gpio_a22_pad;
input gpio_a23_pad;

/* reset pins */
output un_reset_pad;
inout wd_resetn_pad;

/* sd interface */
output en_sd_power_pad;
inout sd_d0_pad;
inout sd_d1_pad;
inout sd_d2_pad;
inout sd_d3_pad;
inout sd_cmd_pad;
output sd_clk_pad /* synthesis syn_maxfan=1 syn_useioff=1 */;

/* led control pins */
output red_led_pad;
output green_led_pad;

/* ethernet led control pins */
output eth_left_ledn_pad;
output eth_right_ledn_pad;

/* rtc pins */
inout rtc_sda_pad;
output rtc_scl_pad;

output nand_csn_pad;
output nand_rdn_pad;
output nand_wrn_pad;
output nand_ale_pad;
output nand_cle_pad;
input nand_busyn_pad;
inout [7:0] nand_d_pad;

assign eth_left_ledn_pad = !gpio_a22_pad;
assign eth_right_ledn_pad = !gpio_a23_pad;

parameter sdcard_opt = 1'b1;
parameter spi_opt = 1'b1;
parameter nandflash_opt = 1'b1;
parameter can_opt = 1'b0;
/* software currently requires these to be enabled/disabled contiguously. */
parameter xuart0_opt = 1'b0;
parameter xuart1_opt = 1'b0;
parameter xuart2_opt = 1'b0;
parameter xuart3_opt = 1'b0;
parameter xuart4_opt = 1'b0;
parameter xuart5_opt = 1'b0;
parameter xuart6_opt = 1'b0;
parameter xuart7_opt = 1'b0;


/****************************************************************************
 * Boilerplate FPGA configuration (clocks, PLL, DLL, reset) 
 ****************************************************************************/
wire pll_75mhz, pll_75mhz_shifted, pll_100mhz;
wire lock, lock2;
wire [4:0] pllphase;
pll clkgen (
  .CLK(fpga_25mhz_pad),
  .CLKOP(pll_75mhz),
  .CLKOS(pll_75mhz_shifted),
  .LOCK(lock),
  .DPHASE3(pllphase[4]),
  .DPHASE2(pllphase[3]),
  .DPHASE1(pllphase[2]),
  .DPHASE0(pllphase[1]),
  .WRDEL(pllphase[0])
);
pll2 clkgen2 (
  .CLK(fpga_25mhz_pad),
  .CLKOP(pll_100mhz),
  .LOCK(lock2)
);
wire wb_rst_o, wb_int_o, wb_int_i;
wire internal_osc;
reg [15:0] rst_counter = 16'd0;
reg rstn = 1'b0;
always @(posedge internal_osc) begin
  if (lock && lock2) rst_counter <= rst_counter + 1'b1;
  else begin
    rst_counter <= 16'd0;
    rstn <= 1'b0;
  end
  if (rst_counter[15]) rstn <= 1'b1;
end
wire rst = !rstn;
wire reboot;
assign wd_resetn_pad = !reboot;
reg [31:0] timer;
reg unreset;
reg spi_boot_done;
wire xuart_irq, can_irqn;
assign gpio_a0_pad = xuart_irq;
assign gpio_a1_pad = (!can_irqn && can_opt);
wire sbus_enabled;
wire [4:0] gpio_a17to13 = {gpio_a17_pad, gpio_a16_pad, gpio_a15_pad,
  2'b00};
assign un_reset_pad = unreset;
wire wb_clk;
reg mode1, mode2, mode3;
always @(posedge pll_75mhz or posedge rst) begin
  if (rst) begin
    timer <= 32'd0;
    unreset <= 1'b0;
    spi_boot_done <= 1'b0;
    mode1 <= 1'b1;
    mode2 <= 1'b1;
    mode3 <= 1'b1;
  end else begin
    timer <= timer + 1'b1;
    if (timer >= 32'd64) begin
      unreset <= 1'b1;
      if (!unreset) begin
        mode1 <= dio_pad[26];
        mode2 <= dio_pad[25];
        mode3 <= gpio_a22_pad;
      end
    end
    if (unreset && gpio_a17to13[4:2] == 3'b110) spi_boot_done <= 1'b1;
  end
end
reg [4:0] seq_24mhz;
reg clk_24mhz;
always @(posedge pll_100mhz or posedge rst) begin
  if (rst) begin
    clk_24mhz <= 1'b1;
    seq_24mhz <= 5'd0;
  end else begin
    seq_24mhz <= seq_24mhz + 1'b1;
    case (seq_24mhz)
    1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21: clk_24mhz <= !clk_24mhz;
    24: begin
      clk_24mhz <= !clk_24mhz;
      seq_24mhz <= 5'd0;
    end
    endcase
  end
end
wire led_red;
assign red_led_pad = !led_red;
assign wb_rst = rst;
wire led_grn;
assign green_led_pad = !led_grn;
wire [7:0] xuart_disable = {!xuart7_opt[0], !xuart6_opt[0], !xuart5_opt[0],
  !xuart4_opt[0], !xuart3_opt[0], !xuart2_opt[0], !xuart1_opt[0],
  !xuart0_opt[0]};
wire xuart_opt = (xuart_disable[7:0] != 8'hff);


/****************************************************************************
 * XUART blockram
 ****************************************************************************/
wire spi_cs0 = gpio_a28_pad;
wire ramwbs1_cyc_i, ramwbs1_stb_i, ramwbs1_we_i, ramwbs1_ack_o;
wire [31:0] ramwbs1_dat_i, ramwbs1_dat_o;
wire [31:0] ramwbs1_adr_i;
wire ramwbs2_cyc_i, ramwbs2_stb_i, ramwbs2_we_i, ramwbs2_ack_o;
wire [31:0] ramwbs2_dat_i, ramwbs2_dat_o;
wire [31:0] ramwbs2_adr_i;
wire [3:0] ramwbs2_sel_i, ramwbs1_sel_i;
assign sbus_enabled = 1'b1;
assign dcs_clk = pll_75mhz;
blockram_8kbyte sramcore(
  .wb1_cyc_i(ramwbs1_cyc_i),
  .wb1_stb_i(ramwbs1_stb_i),
  .wb1_we_i(ramwbs1_we_i),
  .wb1_dat_i(ramwbs1_dat_i),
  .wb1_dat_o(ramwbs1_dat_o),
  .wb1_adr_i(ramwbs1_adr_i),
  .wb1_ack_o(ramwbs1_ack_o),
  .wb1_sel_i(ramwbs1_sel_i),

  .wb2_cyc_i(ramwbs2_cyc_i),
  .wb2_stb_i(ramwbs2_stb_i),
  .wb2_we_i(ramwbs2_we_i),
  .wb2_dat_i(ramwbs2_dat_i),
  .wb2_dat_o(ramwbs2_dat_o),
  .wb2_adr_i(ramwbs2_adr_i),
  .wb2_ack_o(ramwbs2_ack_o),
  .wb2_sel_i(ramwbs2_sel_i),

  .wb_clk_i(dcs_clk && xuart_opt),
  .wb_rst_i(rst)
);


/****************************************************************************
 * SBUS protocol core (SPI -> WISHBONE bridge)
 ****************************************************************************/
wire sbus_so;
wire spiwbm_cyc_o, spiwbm_stb_o, spiwbm_clk_o, spiwbm_we_o;
wire [4:0] spiwbm_adr_o;
wire [15:0] spiwbm_dat_o;
reg spiwbm_ack_i;
reg [15:0] spiwbm_dat_i;
wire [1:0] spiwbm_sel_o;
wire spiwbm2_cyc_o, spiwbm2_stb_o, spiwbm2_clk_o, spiwbm2_we_o;
wire [4:0] spiwbm2_adr_o;
wire [15:0] spiwbm2_dat_o;
reg spiwbm2_ack_i;
reg [15:0] spiwbm2_dat_i;
wire [1:0] spiwbm2_sel_o;
reg spiwbm2_en;
wire spiwbm2_clk;
wire [3:0] spiwbm32_sel_o = 
  spiwbm_adr_o[1] ? {spiwbm_sel_o, 2'b00} : {2'b00, spiwbm_sel_o};
assign wb_clk = dcs_clk;
spi_sbus_resync sbuscore(
  .wbm_clk_i(dcs_clk),
  .wbm_adr_o(spiwbm_adr_o),
  .wbm_dat_i(spiwbm_dat_i),
  .wbm_ack_i(spiwbm_ack_i),
  .wbm_cyc_o(spiwbm_cyc_o),
  .wbm_stb_o(spiwbm_stb_o),
  .wbm_we_o(spiwbm_we_o),
  .wbm_dat_o(spiwbm_dat_o),
  .wbm_sel_o(spiwbm_sel_o),

  .wbm2_clk_o(spiwbm2_clk),
  .wbm2_en_i(spiwbm2_en),
  .wbm2_adr_o(spiwbm2_adr_o),
  .wbm2_dat_i(spiwbm2_dat_i),
  .wbm2_ack_i(spiwbm2_ack_i),
  .wbm2_cyc_o(spiwbm2_cyc_o),
  .wbm2_stb_o(spiwbm2_stb_o),
  .wbm2_we_o(spiwbm2_we_o),
  .wbm2_dat_o(spiwbm2_dat_o),
  .wbm2_sel_o(spiwbm2_sel_o),

  .rst_i(wb_rst),
  .csn_i(spi_cs0 || !sbus_enabled),
  .sck_i(cpu_spi_clk_pad),
  .si_i(cpu_spi_mosi_pad),
  .so_o(sbus_so),
  .sel_i({gpio_a17_pad, gpio_a3_pad})

);
assign cpu_spi_miso_pad = sbus_so;


/****************************************************************************
 * SD card logic
 ****************************************************************************/
reg sdwbs1_en, sdwbs2_en;
wire [31:0] sdwbs1_dat, sdwbs2_dat;
wire [15:0] sdwbs1_dat_o = 
  spiwbm_adr_o[1] ? sdwbs1_dat[31:16] : sdwbs1_dat[15:0];
wire [15:0] sdwbs2_dat_o = sdwbs2_dat[15:0];
wire sdwbs1_ack_o, sdwbs2_ack_o;
wire sd_cmd, sd_cmd_oe;
wire [3:0] sd_dat, sd_dat_oe;
assign sd_cmd_pad = sd_cmd_oe ? sd_cmd : 1'bz;
assign sd_d0_pad = sd_dat_oe[0] ? sd_dat[0] : 1'bz;
assign sd_d1_pad = sd_dat_oe[1] ? sd_dat[1] : 1'bz;
assign sd_d2_pad = sd_dat_oe[2] ? sd_dat[2] : 1'bz;
assign sd_d3_pad = sd_dat_oe[3] ? sd_dat[3] : 1'bz;
wire [7:0] sd_clk;
wire sd_busy, sd_power;
assign en_sd_power_pad = unreset ? sd_power : 1'b1;
ts_sdcore sdcardcore(
  .wb_rst_i(wb_rst),
  .wb_clk_i(wb_clk & sdcard_opt),

  .wb1_adr_i(spiwbm_adr_o),
  .wb1_cyc_i(spiwbm_cyc_o && sdwbs1_en),
  .wb1_stb_i(spiwbm_stb_o && sdwbs1_en),
  .wb1_dat_o(sdwbs1_dat),
  .wb1_we_i(spiwbm_we_o),
  .wb1_dat_i({spiwbm_dat_o, spiwbm_dat_o}),
  .wb1_ack_o(sdwbs1_ack_o),
  .wb1_sel_i(spiwbm32_sel_o),
  .wb1_bte_i(2'b00),
  .wb1_cti_i(3'b000),

  .sd_cmd_i(sd_cmd_pad),
  .sd_cmd_o(sd_cmd),
  .sd_cmd_oe_o(sd_cmd_oe),
  .sd_dat_i({sd_d3_pad, sd_d2_pad, sd_d1_pad, sd_d0_pad}),
  .sd_dat_o(sd_dat),
  .sd_dat_oe_o(sd_dat_oe),
  .sd_clk_o(sd_clk),
 
  .sd_wprot_i(8'b00000000),
  .sd_detect_i(8'b00000001),
  .sd_power_o(sd_power),
  .sd_busy_o(sd_busy)
);
assign sd_clk_pad = sd_clk[0];


/****************************************************************************
 * SPI controller logic
 ****************************************************************************/
reg spiwbs_en;
wire [15:0] spiwbs_dat_o;
wire spiwbs_ack_o;
wire scken;
wire [3:0] csn;
reg [3:0] csn_q;
always @(posedge wb_clk or posedge wb_rst) begin
  if (wb_rst) csn_q <= 4'hf;
  else csn_q <= csn;
end
wire [3:0] si = {4{spi_miso_pad}};
wire sck, hispeed;
assign spi_clk_pad = hispeed ? (pll_75mhz_shifted | !scken) : sck;
wire so;
assign spi_mosi_pad = so;
wb_spi spicore(
  .wb_rst_i(wb_rst),
  .wb_clk_i(wb_clk & spi_opt),
  .wb_cyc_i(spiwbm_cyc_o && spiwbs_en),
  .wb_stb_i(spiwbm_stb_o && spiwbs_en),
  .wb_dat_o(spiwbs_dat_o),
  .wb_dat_i(spiwbm_dat_o),
  .wb_adr_i(spiwbm_adr_o),
  .wb_we_i(spiwbm_we_o),
  .wb_ack_o(spiwbs_ack_o),
  .wb_sel_i(spiwbm_sel_o),

  .scken_o(scken),
  .so_o(so),
  .si_i(si),
  .csn_o(csn),
  .sck_o(sck),
  .hispeed_o(hispeed)
);


/****************************************************************************
 * TS-XUART super uarts
 ****************************************************************************/
wire xuartwbs_ack_o;
reg xuwbs_en;
wire [31:0] uartwbm_adr_o;
wire [15:0] uartwbm_dat_o, xuartwbs_dat_o;
wire [15:0] uartwbm_dat_i;
reg [7:0] xuart_rx;
wire [7:0] xuart_active;
wire [7:0] xuart_cts = 8'hff;
wire [7:0] xuart_txen, xuart_tx;
reg [7:0] xuart_rx_q, xuart_cts_q;
always @(posedge wb_clk or posedge wb_rst) begin
  if (wb_rst) begin
    xuart_rx_q <= 8'hff;
    xuart_cts_q <= 8'hff;
  end else begin
    xuart_rx_q <= xuart_rx;
    xuart_cts_q <= xuart_cts;
  end
end
assign ramwbs1_dat_i[31:0] = {uartwbm_dat_o[15:0], uartwbm_dat_o[15:0]}; 
assign ramwbs1_adr_i = {uartwbm_adr_o[15:2], 2'b00};
assign uartwbm_dat_i[15:0] = 
  uartwbm_adr_o[1] ? ramwbs1_dat_o[31:16] : ramwbs1_dat_o[15:0];
assign ramwbs1_sel_i = uartwbm_adr_o[1] ? 4'b1100 : 4'b0011;
ts_xuart tsxuartcore(
  .wb_clk_i(wb_clk & xuart_opt),
  .wb_rst_i(wb_rst),

  .wbm_cyc_o(ramwbs1_cyc_i),
  .wbm_stb_o(ramwbs1_stb_i),
  .wbm_we_o(ramwbs1_we_i),
  .wbm_ack_i(ramwbs1_ack_o),
  .wbm_adr_o(uartwbm_adr_o),
  .wbm_dat_i(uartwbm_dat_i),
  .wbm_dat_o(uartwbm_dat_o),


  .wbs_cyc_i(spiwbm_cyc_o && xuwbs_en),
  .wbs_stb_i(spiwbm_stb_o && xuwbs_en),
  .wbs_we_i(spiwbm_we_o),
  .wbs_adr_i(spiwbm_adr_o),
  .wbs_dat_i(spiwbm_dat_o),
  .wbs_dat_o(xuartwbs_dat_o),
  .wbs_ack_o(xuartwbs_ack_o),

  .baudclk_i(pll_100mhz),
  .cts_i(xuart_cts_q),
  .rx_i(xuart_rx_q),
  .tx_o(xuart_tx),
  .txen_o(xuart_txen),
  .active_o(xuart_active),
  .irq_o(xuart_irq),
  .uart_disable_i(xuart_disable)
);


/****************************************************************************
 * Syscon (misc control registers)
 ****************************************************************************/
reg scwbs_en;
wire [15:0] scwbs_dat_o;
wire scwbs_ack_o;
wire rtc_sda, rtc_scl, rtc_sda_oe, rtc_scl_oe;
assign rtc_sda_pad = rtc_sda_oe ? rtc_sda : 1'bz;
assign rtc_scl_pad = rtc_scl_oe ? rtc_scl : 1'bz;
wire [52:0] dio, dio_oe;
integer i;
reg [52:0] dio_reg;
assign dio_pad[52:0] = dio_reg[52:0];
wire can_tx, can_enable;

wire xbus_csn, xbus_dir, xbus_ale, xbus_bhen, xbus_mux_oe;
wire [15:0] xbus_dat;
wire [15:0] bus_config;

wire adc_scl, adc_sda, an_sel, adc_active;

always @(*) begin
  for (i = 0; i <= 52; i = i + 1) begin
    dio_reg[i] = dio_oe[i] ? dio[i] : 1'bz;
  end

  /* ADC core overrides DIO 6, 35, and 8 */
  if (adc_active) begin
    dio_reg[35] = adc_scl ? 1'bz : 1'b0;
    dio_reg[6] = adc_sda ? 1'bz : 1'b0;
    dio_reg[8] = an_sel;
  end

  /* SPI overrides DIO#17,48,18,19 for CS# signals */
  if (!csn[0] || !csn_q[0]) dio_reg[17] = csn[0];
  if (!csn[1] || !csn_q[1]) dio_reg[48] = csn[1];
  if (!csn[2] || !csn_q[2]) dio_reg[18] = csn[2];
  if (!csn[3] || !csn_q[3]) dio_reg[19] = csn[3];

  /* DIO#26 and DIO#27 are are latched bootup MODE1/MODE2 pins */
  if (!unreset) dio_reg[26:25] = 2'bzz;

  xuart_rx[0] = dio_pad[37];
  if (xuart_active[0]) begin
    dio_reg[36] = xuart_tx[0];
    dio_reg[37] = 1'bz;
    dio_reg[12] = xuart_txen[0];
  end
  xuart_rx[1] = dio_pad[39];
  if (xuart_active[1]) begin
    dio_reg[39] = 1'bz;
    dio_reg[38] = xuart_tx[1];
    dio_reg[13] = xuart_txen[1];
  end
  xuart_rx[2] = dio_pad[44];
  if (xuart_active[2]) begin
    dio_reg[44] = 1'bz;
    dio_reg[22] = xuart_tx[2];
    dio_reg[14] = xuart_txen[2];
  end
  xuart_rx[3] = dio_pad[46];
  if (xuart_active[3]) begin
    dio_reg[46] = 1'bz;
    dio_reg[45] = xuart_tx[3];
  end
  xuart_rx[4] = dio_pad[40];
  if (xuart_active[4]) begin
    dio_reg[40] = 1'bz;
    dio_reg[47] = xuart_tx[4];
  end
  xuart_rx[5] = dio_pad[42];
  if (xuart_active[5]) begin
    dio_reg[42] = 1'bz;
    dio_reg[41] = xuart_tx[5];
  end
  xuart_rx[6] = dio_pad[11];
  if (xuart_active[6]) begin
    dio_reg[11] = 1'bz;
    dio_reg[10] = xuart_tx[6];
  end
  xuart_rx[7] = dio_pad[28];
  if (xuart_active[7]) begin
    dio_reg[28] = 1'bz;
    dio_reg[29] = xuart_tx[7];
  end

  /* CAN overrides DIO#15 and DIO#16 */
  if (can_enable) begin
    dio_reg[16] = 1'bz;
    dio_reg[15] = can_tx;
  end
  
  /* External bus overrides DIO on bus signals: */
  if (bus_config[0]) begin
    dio_reg[23] = xbus_bhen;
    dio_reg[24] = xbus_csn;
    dio_reg[25] = xbus_dir;
    dio_reg[26] = !xbus_ale; // Pin is ALE#, but ale is active high internally
    // there is on bus_wait line on this board.
    dio_reg[34:27] = xbus_mux_oe ? xbus_dat[15:8] : 8'hzz;
  end

end

syscon sysconcore(
  .wb_clk_i(spiwbm2_clk),
  .wb_rst_i(wb_rst),

  .wb_cyc_i(spiwbm2_cyc_o && scwbs_en),
  .wb_stb_i(spiwbm2_stb_o && scwbs_en),
  .wb_we_i(spiwbm2_we_o),
  .wb_adr_i(spiwbm2_adr_o),
  .wb_dat_i(spiwbm2_dat_o),
  .wb_sel_i(spiwbm2_sel_o),
  .wb_dat_o(scwbs_dat_o),
  .wb_ack_o(scwbs_ack_o),

  .dio_i(dio_pad),
  .dio_oe_o(dio_oe),
  .dio_o(dio),

  .rtc_sda_o(rtc_sda),
  .rtc_sda_i(rtc_sda_pad),
  .rtc_sda_oe_o(rtc_sda_oe),
  .rtc_scl_o(rtc_scl),
  .rtc_scl_i(rtc_scl_pad),
  .rtc_scl_oe_o(rtc_scl_oe),

  .clk_100mhz_i(pll_100mhz),
  .reboot_o(reboot),
  .led_grn_o(led_red),
  .led_red_o(led_grn),
  .mode1_i(mode1),
  .mode2_i(mode2),
  .mode3_i(mode3),
  .pllphase_o(pllphase),
  .internal_osc_o(internal_osc)
);


/****************************************************************************
 * NAND flash controller logic
 ****************************************************************************/
reg nandwbs_en;
wire [31:0] nandwbs_dat;
wire nandwbs_ack_o;
wire [7:0] nand_dat;
wire nand_dat_oe;
wire [15:0] nandwbs_dat_o = spiwbm_adr_o[1] ? nandwbs_dat[31:16] : nandwbs_dat[15:0];
assign nand_d_pad = (bus_config[0] && xbus_mux_oe) ? xbus_dat[7:0] : 
  nand_dat_oe ? nand_dat : 8'hzz;
nandflash nandcore(
  .wb_clk_i(wb_clk & nandflash_opt),
  .wb_rst_i(wb_rst),
  .wb1_cyc_i(spiwbm_cyc_o && nandwbs_en),
  .wb1_stb_i(spiwbm_stb_o && nandwbs_en),
  .wb1_we_i(spiwbm_we_o),
  .wb1_adr_i(spiwbm_adr_o[4:0] & 5'hf),
  .wb1_dat_i({spiwbm_dat_o, spiwbm_dat_o}),
  .wb1_dat_o(nandwbs_dat),
  .wb1_ack_o(nandwbs_ack_o),
  .wb1_sel_i(spiwbm32_sel_o),
  .wb1_bte_i(2'b00),
  .wb1_cti_i(3'b000),

  .nand_cle_o(nand_cle_pad),
  .nand_ale_o(nand_ale_pad),
  .nand_cs_o(nand_csn_pad),
  .nand_rd_o(nand_rdn_pad),
  .nand_wr_o(nand_wrn_pad),
  .nand_busy_i(nand_busyn_pad),
  .nand_dat_i(nand_d_pad),
  .nand_dat_o(nand_dat),
  .nand_dat_oe_o(nand_dat_oe)
);


/****************************************************************************
 * SPI SBUS blockram window (for XUART 8kbyte memory access)
 ****************************************************************************/
reg mwinwbs_en;
wire [15:0] mwinwbs_dat_o;
wire mwinwbs_ack_o;
wb_memwindow16to32 mwincore(
  .wb_clk_i(wb_clk && xuart_opt),
  .wb_rst_i(wb_rst),

  .wb_cyc_i(spiwbm_cyc_o && mwinwbs_en),
  .wb_stb_i(spiwbm_stb_o && mwinwbs_en),
  .wb_we_i(spiwbm_we_o),
  .wb_adr_i(spiwbm_adr_o),
  .wb_dat_i(spiwbm_dat_o),
  .wb_dat_o(mwinwbs_dat_o),
  .wb_ack_o(mwinwbs_ack_o),
  
  .wbm_cyc_o(ramwbs2_cyc_i),
  .wbm_stb_o(ramwbs2_stb_i),
  .wbm_we_o(ramwbs2_we_i),
  .wbm_adr_o(ramwbs2_adr_i),
  .wbm_dat_o(ramwbs2_dat_i),
  .wbm_dat_i(ramwbs2_dat_o),
  .wbm_ack_i(ramwbs2_ack_o),
  .wbm_sel_o(ramwbs2_sel_i)
);


/****************************************************************************
 * SJA1000C compatible CAN controller
 ****************************************************************************/
wire canwbs_cyc_i, canwbs_stb_i, canwbs_we_i, canwbs_ack_o;
wire [7:0] canwbs_dat_i, canwbs_dat_o;
wire [15:0] canwbs_adr_i;
can_top cancore(
  .wb_clk_i(wb_clk && can_opt),
  .wb_rst_i(!unreset),
  .wb_cyc_i(canwbs_cyc_i),
  .wb_stb_i(canwbs_stb_i),
  .wb_we_i(canwbs_we_i),
  .wb_dat_i(canwbs_dat_i),
  .wb_dat_o(canwbs_dat_o),
  .wb_adr_i(canwbs_adr_i[7:0]),
  .wb_ack_o(canwbs_ack_o),

  .clk_i(clk_24mhz && can_opt),
  .rx_i(dio_pad[16]),
  .tx_o(can_tx),
  .irq_on(can_irqn),
  .bus_off_on(can_enable)
);


/****************************************************************************
 * SPI SBUS CAN window (for 256 by 8-bit SJA1000C CAN address space)
 ****************************************************************************/
reg mwinwbs2_en;
wire [15:0] mwinwbs2_dat_o;
wire mwinwbs2_ack_o;
wb_memwindow16to8 mwincore2(
  .wb_clk_i(wb_clk && can_opt),
  .wb_rst_i(!unreset),

  .wb_cyc_i(spiwbm_cyc_o && mwinwbs2_en),
  .wb_stb_i(spiwbm_stb_o && mwinwbs2_en),
  .wb_we_i(spiwbm_we_o),
  .wb_adr_i(spiwbm_adr_o),
  .wb_dat_i(spiwbm_dat_o),
  .wb_dat_o(mwinwbs2_dat_o),
  .wb_ack_o(mwinwbs2_ack_o),
  
  .wbm_cyc_o(canwbs_cyc_i),
  .wbm_stb_o(canwbs_stb_i),
  .wbm_we_o(canwbs_we_i),
  .wbm_adr_o(canwbs_adr_i),
  .wbm_dat_o(canwbs_dat_i),
  .wbm_dat_i(canwbs_dat_o),
  .wbm_ack_i(canwbs_ack_o)
);


/****************************************************************************
 * General purpose memory window for external bus and any custom registers.
 ****************************************************************************/
wire mwinwbm_cyc_o, mwinwbm_stb_o, mwinwbm_we_o;
reg mwinwbm_ack_i; 
wire [31:0] mwinwbm_adr_o;
wire [31:0] mwinwbm_dat_o;
reg [31:0] mwinwbm_dat_i;
wire [3:0] mwinwbm_sel_o;
reg mwinwbs3_en;
wire [15:0] mwinwbs3_dat_o;
wire mwinwbs3_ack_o;
reg [1:0] mwinbuswidth;
wb_memwindow mwincore3(
  .wb_clk_i(wb_clk),
  .wb_rst_i(!unreset),

  .wb_cyc_i(spiwbm_cyc_o && mwinwbs3_en),
  .wb_stb_i(spiwbm_stb_o && mwinwbs3_en),
  .wb_we_i(spiwbm_we_o),
  .wb_adr_i(spiwbm_adr_o),
  .wb_sel_i(spiwbm_sel_o),
  .wb_dat_i(spiwbm_dat_o),
  .wb_dat_o(mwinwbs3_dat_o),
  .wb_ack_o(mwinwbs3_ack_o),

  .wbm_cyc_o(mwinwbm_cyc_o),
  .wbm_stb_o(mwinwbm_stb_o),
  .wbm_we_o(mwinwbm_we_o),
  .wbm_adr_o(mwinwbm_adr_o),
  .wbm_sel_o(mwinwbm_sel_o),
  .wbm_dat_o(mwinwbm_dat_o),
  .wbm_dat_i(mwinwbm_dat_i),
  .wbm_ack_i(mwinwbm_ack_i),
  .wbm_buswidth_i(mwinbuswidth)
);


/****************************************************************************
 * External 16 bit muxed address/data bus
 ****************************************************************************/
reg xbuswbs_en;
wire [15:0] xbuswbs_dat_o;
wire xbuswbs_ack_o;

/* The first 64KB of memwindow space are for using the external bus in 
 * 16 bit mode.  The next 64KB (adr_o[16] == 1) are for the external bus in
 * 8 bit mode.  For 8 bit mode, the memwindow core will emit 8 bit wishbone
 * cycles, but the external bus core expects 16 bit cycles with a wb_sel of
 * 2'b01 or 2'b10.  This is a sensible way of implementing the external bus 
 * on non-sbus designs, and rather than modify it for this situation, I am
 * going to hook up an 8 to 16 bit adapter for the 8 bit space.  This will 
 * allow maximum bandwidth for PC/104 due to memwindow optimization features.
 * Actually since the TS-81xx only supports 8 bit PC/104, maybe we should put
 * an 8 bit memwindow on all macrocontrollers for this purpose.
 * -- GP
 */

wire [15:0] xbuswbs_dat16_o;
wire [31:0] xbuswbs_adr_i;
wire [15:0] xbuswbs_dat_i;
wire [1:0]  xbuswbs_sel_i;

wb_8to16adapter adapter1(
  .wbm_dat_o(mwinwbm_dat_o[15:0]),
  .wbm_dat_i(xbuswbs_dat_o),
  .wbm_adr_o(mwinwbm_adr_o),
  .wbm_sel_o(mwinwbm_sel_o[1:0]),

  .wbs_dat_i(xbuswbs_dat_i),
  .wbs_dat_o(xbuswbs_dat16_o),
  .wbs_adr_i(xbuswbs_adr_i),
  .wbs_sel_i(xbuswbs_sel_i),

  .toggle8bit_i(mwinwbm_adr_o[16])
);

isamuxbus_bridge externalbus(
  .wb_clk_i(wb_clk),
  .wb_rst_i(!unreset),
  .wb_cyc_i(mwinwbm_cyc_o && xbuswbs_en),
  .wb_stb_i(mwinwbm_stb_o && xbuswbs_en),
  .wb_adr_i(xbuswbs_adr_i),
  .wb_dat_i(xbuswbs_dat_i),
  .wb_sel_i(xbuswbs_sel_i),
  .wb_dat_o(xbuswbs_dat16_o),
  .wb_ack_o(xbuswbs_ack_o),
  .wb_we_i(mwinwbm_we_o),

  .bus_config_i(bus_config),
  .cs_8bit_i(mwinwbm_adr_o[16]),

  .csn_o(xbus_csn),
  .dir_o(xbus_dir),
  .ale_o(xbus_ale),
  .bhen_o(xbus_bhen),
  .waitn_i(1'b1), // TS-4500 has no wait line
  .mux_i({dio_pad[34:27], nand_d_pad[7:0]}),
  .mux_o(xbus_dat),
  .mux_oe_o(xbus_mux_oe)
);

/****************************************************************************
 * User core
 ****************************************************************************/
reg userwbs_en;
wire [15:0] userwbs_dat_o;
wire userwbs_ack_o;

user usercore(
  .wb_clk_i(wb_clk),
  .wb_rst_i(!unreset),

  .wb_cyc_i(mwinwbm_cyc_o && userwbs_en),
  .wb_stb_i(mwinwbm_stb_o && userwbs_en),
  .wb_we_i(mwinwbm_we_o),
  .wb_adr_i(mwinwbm_adr_o),
  .wb_dat_i(mwinwbm_dat_o[15:0]),
  .wb_sel_i(mwinwbm_sel_o[1:0]),
  .wb_dat_o(userwbs_dat_o),
  .wb_ack_o(userwbs_ack_o),

  .bus_config_o(bus_config)
);

/****************************************************************************
 * ADC core
 ****************************************************************************/

reg adcwbs_en;
wire [15:0] adcwbs_dat_o;
wire adcwbs_ack_o;

wb_mcp3428 adc(
  .wb_rst_i(wb_rst),
  .wb_clk_i(wb_clk),
  .wb_cyc_i(mwinwbm_cyc_o && adcwbs_en),
  .wb_stb_i(mwinwbm_stb_o && adcwbs_en),
  .wb_dat_o(adcwbs_dat_o),
  .wb_dat_i(mwinwbm_dat_o),
  .wb_adr_i(mwinwbm_adr_o),
  .wb_we_i(mwinwbm_we_o),
  .wb_ack_o(adcwbs_ack_o),
  .wb_sel_i(mwinwbm_sel_o),

  .scl_o(adc_scl),
  .sda_o(adc_sda),
  .sda_i(dio_pad[6]),
  .an_sel_o(an_sel),
  .active_o(adc_active)
);

/****************************************************************************
 * Memory window address decode
 ****************************************************************************/

/* Address decode:
 * 0 to 0xFFFF            16 bit external bus space
 * 0x10000 to 0x1FFFF     8 bit external bus space
 * 0x20000 to 0x200FF     Bus config register and other user registers
 * higher = reserved
 * Memory window currently decodes 27 bits of address.
 *
 * mwinbuswidth settings:
 *     0 - use internal setting
 *     1 - 32 bit
 *     2 - 16 bit
 *     3 - 8 bit
 */

reg [3:0] decodes;
always @(posedge wb_clk or posedge wb_rst) begin
  if (wb_rst) decodes <= 4'b000;
  else if (mwinwbm_adr_o < 32'h00020000) decodes <= 4'b0001;
  else if (mwinwbm_adr_o < 32'h00021000) decodes <= 4'b0010;
  else if (mwinwbm_adr_o < 32'h00022000) decodes <= 4'b0100;
  else decodes <= 4'b1000;
end

always @(*) begin
  xbuswbs_en = 1'b0;
  userwbs_en = 1'b0;
  mwinwbm_dat_i = 32'hxxxxxxxx;
  mwinwbm_ack_i = 1'b1;
  mwinbuswidth = 2'b00;

  if (decodes[0]) begin
    xbuswbs_en = 1'b1;
    mwinwbm_dat_i[15:0] = xbuswbs_dat_o;
    mwinwbm_ack_i = xbuswbs_ack_o;
  end else if (decodes[1]) begin
    userwbs_en = 1'b1;
    mwinwbm_dat_i[15:0] = userwbs_dat_o;
    mwinwbm_ack_i = userwbs_ack_o;
    mwinbuswidth = 2'b10;
  end else if (decodes[2]) begin
    adcwbs_en = 1'b1;
    mwinwbm_dat_i[15:0] = adcwbs_dat_o;
    mwinwbm_ack_i = adcwbs_ack_o;
    mwinbuswidth = 2'b10;
  end 

end

/****************************************************************************
 * SPI SBUS address decode
 ****************************************************************************/
assign gpio_a23_pad = 1'bz;
assign gpio_a22_pad = 1'bz;
always @(*) begin
  sdwbs1_en = 1'b0;
  spiwbs_en = 1'b0;
  scwbs_en = 1'b0;
  nandwbs_en = 1'b0;
  spiwbm2_en = 1'b0;
  mwinwbs_en = 1'b0;
  xuwbs_en = 1'b0;
  mwinwbs2_en = 1'b0;
  mwinwbs3_en = 1'b0;

  spiwbm_dat_i = 16'hxxxx;
  spiwbm_ack_i = 1'b1;
  spiwbm2_dat_i = 16'hxxxx;
  spiwbm2_ack_i = 1'b1;

  /*
   * Top level address decode:
   *
   * 0x0 - SD card
   * 0x10 - NAND flash controller
   * 0x18 - general purpose memory window
   * 0x20 - XUART/XUART memwindow
   * 0x40 - SPI interface
   * 0x50 - CAN controller interface
   * 0x60 - syscon
   *
   * 4 bits of address come from the SBUS interface, 2 bits of address come
   * from GPIO A16 and A15 and each address represents 16-bit registers.  Total
   * 128 bytes of address space (64 16-bit regs)
   *
   * It is possible to 8-bit writes via GPIO A17 and A3 which act as active high
   * byte lane selects.
   */
  case ({gpio_a16_pad, gpio_a15_pad})
  2'd0: begin
    if (spiwbm_adr_o[4:0] >= 5'h18) begin
      mwinwbs3_en = 1'b1;
      spiwbm_dat_i = mwinwbs3_dat_o;
      spiwbm_ack_i = mwinwbs3_ack_o;
    end else if (spiwbm_adr_o[4:0] >= 5'h10) begin
      nandwbs_en = 1'b1;
      spiwbm_dat_i = nandwbs_dat_o;
      spiwbm_ack_i = nandwbs_ack_o;
    end else begin
      sdwbs1_en = 1'b1;
      spiwbm_dat_i = sdwbs1_dat_o;
      spiwbm_ack_i = sdwbs1_ack_o;
    end
  end
  2'd1: begin
    if (spiwbm_adr_o[4:0] >= 5'h1c) begin
      mwinwbs_en = 1'b1;
      spiwbm_dat_i = mwinwbs_dat_o;
      spiwbm_ack_i = mwinwbs_ack_o;
    end else begin
      xuwbs_en = 1'b1;
      spiwbm_dat_i = xuartwbs_dat_o;
      spiwbm_ack_i = xuartwbs_ack_o;
    end
  end
  2'd2: begin
    if (spiwbm_adr_o[4:0] >= 5'h10) begin
      mwinwbs2_en = 1'b1;
      spiwbm_dat_i = mwinwbs2_dat_o;
      spiwbm_ack_i = mwinwbs2_ack_o;
    end else begin
      spiwbs_en = 1'b1;
      spiwbm_dat_i = spiwbs_dat_o;
      spiwbm_ack_i = spiwbs_ack_o;
    end
  end
  2'd3: begin
    scwbs_en = 1'b1;
    spiwbm2_en = 1'b1;
    spiwbm2_dat_i = scwbs_dat_o;
    spiwbm2_ack_i = scwbs_ack_o;
/*
    spiwbm_dat_i = scwbs_dat_o;
    spiwbm_ack_i = scwbs_ack_o;
*/
  end
  default: begin
    spiwbm_dat_i = 16'hxxxx;
    spiwbm_ack_i = 1'bx;
  end
  endcase
end


endmodule

/* 8to16adapter does one of two things:
 * If toggle8bit_i is 0, it passes the 16 bit wishbone bus straight through.
 * Otherwise, it converts wbm 8 bit cycles into wbs 16 bit cycles.
 */

module wb_8to16adapter (
  wbm_dat_o,
  wbm_dat_i,
  wbm_adr_o,
  wbm_sel_o,

  wbs_dat_i,
  wbs_dat_o,
  wbs_adr_i,
  wbs_sel_i,

  toggle8bit_i
);

input [15:0] wbm_dat_o;
output [15:0] wbm_dat_i;
input [31:0] wbm_adr_o;
input [1:0] wbm_sel_o;

output [15:0] wbs_dat_i;
input [15:0] wbs_dat_o;
output [31:0] wbs_adr_i;
output [1:0] wbs_sel_i;

input toggle8bit_i;

wire t = toggle8bit_i;
wire e = !wbm_adr_o[0];

assign wbm_dat_i = (t && !e) ? {8'hxx, wbs_dat_o[15:8]} : wbs_dat_o;
assign wbs_dat_i = t ? {wbm_dat_o[7:0], wbm_dat_o[7:0]} : wbm_dat_o;
assign wbs_adr_i[31:1] = wbm_adr_o[31:1];
assign wbs_adr_i[0] = t ? 1'b0 : wbm_adr_o[0];
assign wbs_sel_i = t ? e ? 2'b01 : 2'b10 : wbm_sel_o;

endmodule
